Package: universalmotif
Title: Import, Modify, and Export Motifs with R
Version: 1.31.46
Authors@R: c(
      person(c("Benjamin", "Jean-Marie"), "Tremblay", role = c("aut", "cre"),
        email = "benjamin.tremblay@uwaterloo.ca",
        comment = c(ORCID = "0000-0002-7441-2951")),
      person("Spencer", "Nystrom", role = "ctb",
        comment = c(ORCID = "0000-0003-1000-1579"))
    )
URL: https://bioconductor.org/packages/universalmotif/
BugReports: https://github.com/bjmt/universalmotif/issues
Description: A comprehensive toolkit for working with sequence motifs
        in R. Imports and exports most common motif formats (JASPAR,
        MEME, HOMER, TRANSFAC, CIS-BP, UNIPROBE) and interoperates with
        the other Bioconductor motif packages. Analysis functions cover
        de novo motif discovery, motif-vs-motif comparison and
        clustering, P-value calculation, sequence scanning, enrichment
        against shuffled or composition-matched backgrounds, positional
        bias testing, and pairwise motif co-occurrence. Also includes
        utilities for sequence shuffling, motif trimming, higher-order
        representations, ground-truth simulation by motif implantation,
        and logo-plotting functionality.
Depends: R (>= 4.1.0)
License: GPL-3
Encoding: UTF-8
Imports: methods, stats, utils, parallel, MASS, ggplot2, yaml, IRanges,
        Rcpp, Biostrings, BiocGenerics, S4Vectors, rlang, grid,
        grDevices, MatrixGenerics
Suggests: spelling, knitr, bookdown, TFBSTools, rmarkdown, MotifDb,
        testthat, BiocParallel, seqLogo, motifStack, dplyr, ape,
        ggtree, processx, ggseqlogo, cowplot, GenomicRanges,
        GenomeInfoDb, ggbio, rtracklayer, BSgenome,
        BSgenome.Athaliana.TAIR.TAIR9
Enhances: PWMEnrich, rGADEM
LinkingTo: Rcpp, RcppThread
VignetteBuilder: knitr
biocViews: MotifAnnotation, MotifDiscovery, DataImport, GeneRegulation
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE, old_usage = TRUE)
Language: en-GB
Collate: 'RcppExports.R' 'add_multifreq.R' 'compare_motifs.R'
        'compare_motifs_lite.R' 'compare_report.R'
        'universalmotif-class.R' 'convert_motifs.R' 'convert_type.R'
        'create_motif.R' 'create_sequences.R' 'data.R' 'dedup_hits.R'
        'enrich_motifs.R' 'enrich_motifs_lite.R' 'filter_motifs.R'
        'geom_motif.R' 'get_bkg.R' 'implant_motifs.R' 'make_DBscores.R'
        'match_bkg.R' 'merge_motifs.R' 'merge_motifs_lite.R'
        'merge_similar.R' 'merge_similar_lite.R' 'motif_coocc.R'
        'motif_finder.R' 'motif_peaks.R' 'motif_proximity.R'
        'motif_pvalue.R' 'motif_rc.R' 'motif_tree.R'
        'motif_tree_lite.R' 'read_cisbp.R' 'read_homer.R'
        'read_jaspar.R' 'read_matrix.R' 'read_meme.R' 'read_motifs.R'
        'read_transfac.R' 'read_uniprobe.R' 'run_meme.R'
        'sample_sites.R' 'scan_sequences.R' 'scan_sequences_lite.R'
        'sequence_complexity.R' 'shuffle_motifs.R'
        'shuffle_sequences.R' 'switch_alph.R' 'trim_cwm.R'
        'trim_motifs.R' 'universalmotif-methods.R' 'universalmotif.R'
        'universalmotif_df.R' 'utils-internal.R' 'utils-motif.R'
        'utils-sequence.R' 'view_logo.R' 'view_motifs.R'
        'view_motifs_lite.R' 'write_homer.R' 'write_jaspar.R'
        'write_matrix.R' 'write_meme.R' 'write_motifs.R'
        'write_transfac.R' 'zzz.R'
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-06-25 15:20:29 UTC
RemoteUrl: https://github.com/bioc/universalmotif
RemoteRef: HEAD
RemoteSha: b04fa55cc2ff868d77a4f81e47051b82a659bdf6
NeedsCompilation: yes
Packaged: 2026-06-25 17:22:52 UTC; root
Author: Benjamin Jean-Marie Tremblay [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7441-2951>),
  Spencer Nystrom [ctb] (ORCID: <https://orcid.org/0000-0003-1000-1579>)
Maintainer: Benjamin Jean-Marie Tremblay <benjamin.tremblay@uwaterloo.ca>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-25 17:28:07 UTC; unix
