# Generated by roxygen2: do not edit by hand

S3method(as.character,universalmotif)
S3method(c,universalmotif)
S3method(print,universalmotif_df)
export(add_gap)
export(add_multifreq)
export(as.data.frame)
export(average_ic)
export(calc_complexity)
export(calc_windows)
export(cbind)
export(colMeans)
export(colSums)
export(colnames)
export(compare_columns)
export(compare_motifs)
export(consensus_to_ppm)
export(consensus_to_ppmAA)
export(convert_motifs)
export(convert_type)
export(count_klets)
export(create_motif)
export(create_sequences)
export(enrich_motifs)
export(filter_motifs)
export(get_bkg)
export(get_consensus)
export(get_consensusAA)
export(get_klets)
export(get_matches)
export(get_scores)
export(icm_to_ppm)
export(log_string_pval)
export(make_DBscores)
export(mask_ranges)
export(mask_seqs)
export(meme_alph)
export(merge_motifs)
export(merge_similar)
export(motif_peaks)
export(motif_pvalue)
export(motif_range)
export(motif_rc)
export(motif_score)
export(motif_tree)
export(ncol)
export(normalize)
export(nrow)
export(pcm_to_ppm)
export(position_icscore)
export(ppm_to_icm)
export(ppm_to_pcm)
export(ppm_to_pwm)
export(prob_match)
export(prob_match_bkg)
export(pwm_to_ppm)
export(read_cisbp)
export(read_homer)
export(read_jaspar)
export(read_matrix)
export(read_meme)
export(read_motifs)
export(read_transfac)
export(read_uniprobe)
export(requires_update)
export(round_motif)
export(rowMeans)
export(rowSums)
export(rownames)
export(run_meme)
export(sample_sites)
export(scan_sequences)
export(score_match)
export(sequence_complexity)
export(shuffle_motifs)
export(shuffle_sequences)
export(shuffle_string)
export(slide_fun)
export(subset)
export(summarise_motifs)
export(switch_alph)
export(to_df)
export(to_list)
export(trim_motifs)
export(ungap)
export(update_motifs)
export(view_logo)
export(view_motifs)
export(window_string)
export(write_homer)
export(write_jaspar)
export(write_matrix)
export(write_meme)
export(write_motifs)
export(write_transfac)
exportClasses(universalmotif)
exportMethods(convert_motifs)
exportMethods(create_motif)
importFrom(BiocGenerics,as.data.frame)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,ncol)
importFrom(BiocGenerics,normalize)
importFrom(BiocGenerics,nrow)
importFrom(BiocGenerics,rownames)
importFrom(BiocGenerics,subset)
importFrom(Biostrings,AAString)
importFrom(Biostrings,AAStringSet)
importFrom(Biostrings,AA_ALPHABET)
importFrom(Biostrings,AA_STANDARD)
importFrom(Biostrings,BString)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,DNA_ALPHABET)
importFrom(Biostrings,DNA_BASES)
importFrom(Biostrings,PWM)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,RNA_BASES)
importFrom(Biostrings,as.matrix)
importFrom(Biostrings,consensusMatrix)
importFrom(Biostrings,dinucleotideFrequency)
importFrom(Biostrings,injectHardMask)
importFrom(Biostrings,mask)
importFrom(Biostrings,matchPWM)
importFrom(Biostrings,oligonucleotideFrequency)
importFrom(Biostrings,oligonucleotideTransitions)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,seqtype)
importFrom(Biostrings,trinucleotideFrequency)
importFrom(Biostrings,width)
importFrom(Biostrings,writeXStringSet)
importFrom(IRanges,IRanges)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,stack)
importFrom(MASS,fitdistr)
importFrom(MatrixGenerics,colMeans)
importFrom(MatrixGenerics,colSums)
importFrom(MatrixGenerics,rowMeans)
importFrom(MatrixGenerics,rowSums)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,aggregate)
importFrom(S4Vectors,isTRUEorFALSE)
importFrom(S4Vectors,safeExplode)
importFrom(S4Vectors,wmsg)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grid,unit)
importFrom(methods,"slot<-")
importFrom(methods,as)
importFrom(methods,getClass)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,validObject)
importFrom(rlang,.data)
importFrom(stats,as.dist)
importFrom(stats,chisq.test)
importFrom(stats,cutree)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,fft)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,ks.test)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(stats,xtabs)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,menu)
importFrom(utils,packageDescription)
importFrom(utils,read.table)
importFrom(yaml,as.yaml)
importFrom(yaml,yaml.load)
importMethodsFrom(methods,initialize)
importMethodsFrom(methods,show)
useDynLib(universalmotif)
