---
title: "Introduction to \"universalmotif\""
shorttitle: "Intro to \"universalmotif\""
date: 17 October 2021
author:
- Benjamin Jean-Marie Tremblay^[benjamin.tremblay@uwaterloo.ca]
bibliography: universalmotif.bib
vignette: >
  %\VignetteIndexEntry{Introduction to "universalmotif"}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  bookdown::pdf_document2:
    toc: false
---

```{r setup, echo=FALSE}
knitr::opts_chunk$set(collapse=TRUE, comment = "#>")
```

# Installation {.unnumbered}

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("universalmotif")
```

# Overview {.unnumbered}

For a brief explanation of sequence motifs, see the [Introduction to sequence motifs](IntroductionToSequenceMotifs.pdf) vignette. This broadly covers the different 'types' of motif representation, as well as establishes the nomenclature used by the `universalmotif` package.

The capabilities of the `universalmotif` package can be divided into several general categories. These are briefly demonstrated in the following vignettes:

* [Motif import, export, and manipulation](MotifManipulation.pdf)
* [Sequence scanning and manipulation](SequenceSearches.pdf)
* [Motif comparisons and P-values](MotifComparisonAndPvalues.pdf)

