% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ArabidopsisPromoters}
\alias{ArabidopsisPromoters}
\title{Arabidopsis promoters as a \code{DNAStringSet}.}
\format{
\code{\link{DNAStringSet}}
}
\usage{
ArabidopsisPromoters
}
\description{
50 Arabidopsis promoters, each 1000 bases long. See the
"Sequence manipulation and scanning"
vignette for an example workflow describing extracting promoter
sequences.
}
\keyword{datasets}
